window.Gleam.initSearch([{"doc":"pog","title":"pog","content":"# Pog\n\nA PostgreSQL database client for Gleam, based on [PGO][erlang-pgo].\n\n[erlang-pgo]: https://github.com/erleans/pgo\n\n```gleam\nimport pog\nimport gleam/dynamic\nimport gleeunit/should\n\npub fn main() {\n  // Start a database connection pool.\n  // Typically you will want to create one pool for use in your program\n  let db =\n    pog.default_config()\n    |&gt; pog.host(&quot;localhost&quot;)\n    |&gt; pog.database(&quot;my_database&quot;)\n    |&gt; pog.pool_size(15)\n    |&gt; pog.connect\n\n  // An SQL statement to run. It takes one int as a parameter\n  let sql_query = &quot;\n  select\n    name, age, colour, friends\n  from\n    cats\n  where\n    id = $1&quot;\n\n  // This is the decoder for the value returned by the query\n  let row_decoder = dynamic.tuple4(\n    dynamic.string,\n    dynamic.int,\n    dynamic.string,\n    dynamic.list(dynamic.string),\n  )\n\n  // Run the query against the PostgreSQL database\n  // The int `1` is given as a parameter\n  let assert Ok(response) =\n    pog.query(sql_query)\n    |&gt; pog.parameter(pog.int(1))\n    |&gt; pog.returning(row_decoder)\n    |&gt; pog.execute(db)\n\n  // And then do something with the returned results\n  response.count\n  |&gt; should.equal(2)\n  response.rows\n  |&gt; should.equal([\n    #(&quot;Nubi&quot;, 3, &quot;black&quot;, [&quot;Al&quot;, &quot;Cutlass&quot;]),\n  ])\n}\n```\n\n## Installation\n\n```sh\ngleam add pog\n```\n\n## Support of connection URI\n\nConfiguring a Postgres connection is done by using `Config` type in `pog`.\nTo facilitate connection, and to provide easy integration with the rest of the\nPostgres ecosystem, `pog` provides handling of\n[connection URI as defined by Postgres](https://www.postgresql.org/docs/current/libpq-connect.html#LIBPQ-CONNSTRING-URIS).\nShape of connection URI is `postgresql://[username:password@][host:port][/dbname][?query]`.\nCall `pog.url_config` with your connection URI, and in case it&#39;s correct\nagainst the Postgres standard, your `Config` will be automatically generated!\n\nHere&#39;s an example, using [`envoy`](https://github.com/lpil/envoy) to read the\nconnection URI from the environment.\n\n```gleam\nimport envoy\nimport pog\n\n/// Read the DATABASE_URL environment variable.\n/// Generate the pog.Config from that database URL.\n/// Finally, connect to database.\npub fn read_connection_uri() -&gt; Result(pog.Connection, Nil) {\n  use database_url &lt;- result.try(envoy.get(&quot;DATABASE_URL&quot;))\n  use config &lt;- result.try(pog.url_config(database_url))\n  Ok(pog.connect(config))\n}\n```\n\n## About JSON\n\nIn Postgres, you can define a type `json` or `jsonb`. Such a type can be query\nin SQL, but Postgres returns it a simple string, and accepts it as a simple string!\nWhen writing or reading a JSON, you can simply use\n`pog.text(json.to_string(my_json))` and `dynamic.string` to respectively write\nand read them!\n\n## Rows as maps\n\nBy default, `pgo` will return every selected value from your query as a tuple.\nIn case you want a different output, you can activate `rows_as_maps` in `Config`.\nOnce activated, every returned rows will take the form of a `Dict`.\n\n## Atom generation\n\nCreating a connection pool with the `pog.connect` function dynamically generates\nan Erlang atom. Atoms are not garbage collected and only a certain number of\nthem can exist in an Erlang VM instance, and hitting this limit will result in\nthe VM crashing. Due to this limitation you should not dynamically open new\nconnection pools, instead create the pools you need when your application starts\nand reuse them throughout the lifetime of your program.\n\n## History\n\nPreviously this library was named `gleam_pgo`. This old name is deprecated and\nall future development and support will happen here.\n","url":"index.html"},{"doc":"pog","title":"Config","content":"pub type Config {\n  Config(\n    host: String,\n    port: Int,\n    database: String,\n    user: String,\n    password: Option(String),\n    ssl: Bool,\n    connection_parameters: List(#(String, String)),\n    pool_size: Int,\n    queue_target: Int,\n    queue_interval: Int,\n    idle_interval: Int,\n    trace: Bool,\n    ip_version: IpVersion,\n    rows_as_map: Bool,\n  )\n}\n The configuration for a pool of connections.\nConfig(\n  host: String,\n  port: Int,\n  database: String,\n  user: String,\n  password: Option(String),\n  ssl: Bool,\n  connection_parameters: List(#(String, String)),\n  pool_size: Int,\n  queue_target: Int,\n  queue_interval: Int,\n  idle_interval: Int,\n  trace: Bool,\n  ip_version: IpVersion,\n  rows_as_map: Bool,\n)\n\nhost\n&lt;p&gt;(default: 127.0.0.1): Database server hostname.&lt;/p&gt;\n\nport\n&lt;p&gt;(default: 5432): Port the server is listening on.&lt;/p&gt;\n\ndatabase\n&lt;p&gt;Name of database to use.&lt;/p&gt;\n\nuser\n&lt;p&gt;Username to connect to database as.&lt;/p&gt;\n\npassword\n&lt;p&gt;Password for the user.&lt;/p&gt;\n\nssl\n&lt;p&gt;(default: False): Whether to use SSL or not.&lt;/p&gt;\n\nconnection_parameters\n&lt;p&gt;(default: []): List of 2-tuples, where key and value must be binary\nstrings. You can include any Postgres connection parameter here, such as\n&lt;code&gt;#(&quot;application_name&quot;, &quot;myappname&quot;)&lt;/code&gt; and &lt;code&gt;#(&quot;timezone&quot;, &quot;GMT&quot;)&lt;/code&gt;.&lt;/p&gt;\n\npool_size\n&lt;p&gt;(default: 1): Number of connections to keep open with the database&lt;/p&gt;\n\nqueue_target\n&lt;p&gt;(default: 50) Checking out connections is handled through a queue. If it\ntakes longer than queue_target to get out of the queue for longer than\nqueue_interval then the queue_target will be doubled and checkouts will\nstart to be dropped if that target is surpassed.&lt;/p&gt;\n\nqueue_interval\n&lt;p&gt;(default: 1000)&lt;/p&gt;\n\nidle_interval\n&lt;p&gt;(default: 1000): The database is pinged every idle_interval when the\nconnection is idle.&lt;/p&gt;\n\ntrace\n&lt;p&gt;trace (default: False): pgo is instrumented with &lt;a href=&quot;https://opencensus.io/&quot;&gt;OpenCensus&lt;/a&gt; and\nwhen this option is true a span will be created (if sampled).&lt;/p&gt;\n\nip_version\n&lt;p&gt;(default: Ipv4) Which internet protocol to use for this connection&lt;/p&gt;\n\nrows_as_map\n&lt;p&gt;(default: False) By default, pgo will return a n-tuple, in the order of the query.\nBy setting &lt;code&gt;rows_as_map&lt;/code&gt; to &lt;code&gt;True&lt;/code&gt;, the result will be &lt;code&gt;Dict&lt;/code&gt;.&lt;/p&gt;\n\nSynonyms:\npog.Config\npog Config","url":"pog.html#Config"},{"doc":"pog","title":"Connection","content":"pub type Connection\n A pool of one or more database connections against which queries can be\n made.\n\n Created using the `connect` function and shut-down with the `disconnect`\n function.\n\nSynonyms:\npog.Connection\npog Connection","url":"pog.html#Connection"},{"doc":"pog","title":"Date","content":"pub type Date {\n  Date(year: Int, month: Int, day: Int)\n}\n\nDate(year: Int, month: Int, day: Int)\n\n\nSynonyms:\npog.Date\npog Date","url":"pog.html#Date"},{"doc":"pog","title":"IpVersion","content":"pub type IpVersion {\n  Ipv4\n  Ipv6\n}\n The internet protocol version to use.\nIpv4\n Internet Protocol version 4 (IPv4)\n\nIpv6\n Internet Protocol version 6 (IPv6)\n\nSynonyms:\npog.IpVersion\npog IpVersion","url":"pog.html#IpVersion"},{"doc":"pog","title":"Query","content":"pub opaque type Query(row_type)\n\n\nSynonyms:\npog.Query\npog Query","url":"pog.html#Query"},{"doc":"pog","title":"QueryError","content":"pub type QueryError {\n  ConstraintViolated(\n    message: String,\n    constraint: String,\n    detail: String,\n  )\n  PostgresqlError(code: String, name: String, message: String)\n  UnexpectedArgumentCount(expected: Int, got: Int)\n  UnexpectedArgumentType(expected: String, got: String)\n  UnexpectedResultType(DecodeErrors)\n  ConnectionUnavailable\n}\n\nConstraintViolated(\n  message: String,\n  constraint: String,\n  detail: String,\n)\n The query failed as a database constraint would have been violated by the\n change.\n\nPostgresqlError(code: String, name: String, message: String)\n The query failed within the database.\n https://www.postgresql.org/docs/current/errcodes-appendix.html\n\nUnexpectedArgumentCount(expected: Int, got: Int)\n\n\nUnexpectedArgumentType(expected: String, got: String)\n One of the arguments supplied was not of the type that the query required.\n\nUnexpectedResultType(DecodeErrors)\n The rows returned by the database could not be decoded using the supplied\n dynamic decoder.\n\nConnectionUnavailable\n No connection was available to execute the query. This may be due to\n invalid connection details such as an invalid username or password.\n\nSynonyms:\npog.QueryError\npog QueryError","url":"pog.html#QueryError"},{"doc":"pog","title":"Returned","content":"pub type Returned(t) {\n  Returned(count: Int, rows: List(t))\n}\n The rows and number of rows that are returned by a database query.\nReturned(count: Int, rows: List(t))\n\n\nSynonyms:\npog.Returned\npog Returned","url":"pog.html#Returned"},{"doc":"pog","title":"Time","content":"pub type Time {\n  Time(hours: Int, minutes: Int, seconds: Int, microseconds: Int)\n}\n\nTime(hours: Int, minutes: Int, seconds: Int, microseconds: Int)\n\n\nSynonyms:\npog.Time\npog Time","url":"pog.html#Time"},{"doc":"pog","title":"Timestamp","content":"pub type Timestamp {\n  Timestamp(date: Date, time: Time)\n}\n\nTimestamp(date: Date, time: Time)\n\n\nSynonyms:\npog.Timestamp\npog Timestamp","url":"pog.html#Timestamp"},{"doc":"pog","title":"TransactionError","content":"pub type TransactionError {\n  TransactionQueryError(QueryError)\n  TransactionRolledBack(String)\n}\n\nTransactionQueryError(QueryError)\n\n\nTransactionRolledBack(String)\n\n\nSynonyms:\npog.TransactionError\npog TransactionError","url":"pog.html#TransactionError"},{"doc":"pog","title":"Value","content":"pub type Value\n A value that can be sent to PostgreSQL as one of the arguments to a\n parameterised SQL query.\n\nSynonyms:\npog.Value\npog Value","url":"pog.html#Value"},{"doc":"pog","title":"array","content":"pub fn array(converter: fn(a) -&gt; Value, values: List(a)) -&gt; Value\n\nSynonyms:\npog.array\npog array","url":"pog.html#array"},{"doc":"pog","title":"bool","content":"pub fn bool(a: Bool) -&gt; Value\n\nSynonyms:\npog.bool\npog bool","url":"pog.html#bool"},{"doc":"pog","title":"bytea","content":"pub fn bytea(a: BitArray) -&gt; Value\n\nSynonyms:\npog.bytea\npog bytea","url":"pog.html#bytea"},{"doc":"pog","title":"connect","content":"pub fn connect(a: Config) -&gt; Connection\n Start a database connection pool.\n\n The pool is started in a new process and will asynchronously connect to the\n PostgreSQL instance specified in the config. If the configuration is invalid\n or it cannot connect for another reason it will continue to attempt to\n connect, and any queries made using the connection pool will fail.\nSynonyms:\npog.connect\npog connect","url":"pog.html#connect"},{"doc":"pog","title":"connection_parameter","content":"pub fn connection_parameter(\n  config: Config,\n  name name: String,\n  value value: String,\n) -&gt; Config\n Any Postgres connection parameter here, such as\n `&quot;application_name: myappname&quot;` and `&quot;timezone: GMT&quot;`\nSynonyms:\npog.connection_parameter\npog connection_parameter","url":"pog.html#connection_parameter"},{"doc":"pog","title":"database","content":"pub fn database(config: Config, database: String) -&gt; Config\n Name of database to use.\nSynonyms:\npog.database\npog database","url":"pog.html#database"},{"doc":"pog","title":"date","content":"pub fn date(date: Date) -&gt; Value\n\nSynonyms:\npog.date\npog date","url":"pog.html#date"},{"doc":"pog","title":"decode_date","content":"pub fn decode_date(\n  value: Dynamic,\n) -&gt; Result(Date, List(DecodeError))\n\nSynonyms:\npog.decode_date\npog decode_date","url":"pog.html#decode_date"},{"doc":"pog","title":"decode_time","content":"pub fn decode_time(\n  value: Dynamic,\n) -&gt; Result(Time, List(DecodeError))\n\nSynonyms:\npog.decode_time\npog decode_time","url":"pog.html#decode_time"},{"doc":"pog","title":"decode_timestamp","content":"pub fn decode_timestamp(\n  value: Dynamic,\n) -&gt; Result(Timestamp, List(DecodeError))\n\nSynonyms:\npog.decode_timestamp\npog decode_timestamp","url":"pog.html#decode_timestamp"},{"doc":"pog","title":"default_config","content":"pub fn default_config() -&gt; Config\n The default configuration for a connection pool, with a single connection.\n You will likely want to increase the size of the pool for your application.\n\nSynonyms:\npog.default_config\npog default_config","url":"pog.html#default_config"},{"doc":"pog","title":"disconnect","content":"pub fn disconnect(a: Connection) -&gt; Nil\n Shut down a connection pool.\nSynonyms:\npog.disconnect\npog disconnect","url":"pog.html#disconnect"},{"doc":"pog","title":"error_code_name","content":"pub fn error_code_name(error_code: String) -&gt; Result(String, Nil)\n Get the name for a PostgreSQL error code.\n\n \n &gt; error_code_name(&quot;01007&quot;)\n Ok(&quot;privilege_not_granted&quot;)\n \n\n https://www.postgresql.org/docs/current/errcodes-appendix.html\nSynonyms:\npog.error_code_name\npog error_code_name","url":"pog.html#error_code_name"},{"doc":"pog","title":"execute","content":"pub fn execute(\n  query query: Query(a),\n  on pool: Connection,\n) -&gt; Result(Returned(a), QueryError)\n Run a query against a PostgreSQL database.\n\n The provided dynamic decoder is used to decode the rows returned by\n PostgreSQL. If you are not interested in any returned rows you may want to\n use the `dynamic.dynamic` decoder.\n\nSynonyms:\npog.execute\npog execute","url":"pog.html#execute"},{"doc":"pog","title":"float","content":"pub fn float(a: Float) -&gt; Value\n\nSynonyms:\npog.float\npog float","url":"pog.html#float"},{"doc":"pog","title":"host","content":"pub fn host(config: Config, host: String) -&gt; Config\n Database server hostname.\n\n (default: 127.0.0.1)\nSynonyms:\npog.host\npog host","url":"pog.html#host"},{"doc":"pog","title":"idle_interval","content":"pub fn idle_interval(\n  config: Config,\n  idle_interval: Int,\n) -&gt; Config\n The database is pinged every idle_interval when the connection is idle.\n\n default: 1000\nSynonyms:\npog.idle_interval\npog idle_interval","url":"pog.html#idle_interval"},{"doc":"pog","title":"int","content":"pub fn int(a: Int) -&gt; Value\n\nSynonyms:\npog.int\npog int","url":"pog.html#int"},{"doc":"pog","title":"ip_version","content":"pub fn ip_version(\n  config: Config,\n  ip_version: IpVersion,\n) -&gt; Config\n Which internet protocol to use for this connection\nSynonyms:\npog.ip_version\npog ip_version","url":"pog.html#ip_version"},{"doc":"pog","title":"null","content":"pub fn null() -&gt; Value\n\nSynonyms:\npog.null\npog null","url":"pog.html#null"},{"doc":"pog","title":"nullable","content":"pub fn nullable(\n  inner_type: fn(a) -&gt; Value,\n  value: Option(a),\n) -&gt; Value\n\nSynonyms:\npog.nullable\npog nullable","url":"pog.html#nullable"},{"doc":"pog","title":"parameter","content":"pub fn parameter(query: Query(a), parameter: Value) -&gt; Query(a)\n Push a new query parameter value for the query.\nSynonyms:\npog.parameter\npog parameter","url":"pog.html#parameter"},{"doc":"pog","title":"password","content":"pub fn password(\n  config: Config,\n  password: Option(String),\n) -&gt; Config\n Password for the user.\nSynonyms:\npog.password\npog password","url":"pog.html#password"},{"doc":"pog","title":"pool_size","content":"pub fn pool_size(config: Config, pool_size: Int) -&gt; Config\n Number of connections to keep open with the database\n\n default: 10\nSynonyms:\npog.pool_size\npog pool_size","url":"pog.html#pool_size"},{"doc":"pog","title":"port","content":"pub fn port(config: Config, port: Int) -&gt; Config\n Port the server is listening on.\n\n (default: 5432)\nSynonyms:\npog.port\npog port","url":"pog.html#port"},{"doc":"pog","title":"query","content":"pub fn query(sql: String) -&gt; Query(Nil)\n Create a new query to use with the `execute`, `returning`, and `parameter`\n functions.\n\nSynonyms:\npog.query\npog query","url":"pog.html#query"},{"doc":"pog","title":"queue_interval","content":"pub fn queue_interval(\n  config: Config,\n  queue_interval: Int,\n) -&gt; Config\n Checking out connections is handled through a queue. If it\n takes longer than queue_target to get out of the queue for longer than\n queue_interval then the queue_target will be doubled and checkouts will\n start to be dropped if that target is surpassed.\n\n default: 1000\nSynonyms:\npog.queue_interval\npog queue_interval","url":"pog.html#queue_interval"},{"doc":"pog","title":"queue_target","content":"pub fn queue_target(config: Config, queue_target: Int) -&gt; Config\n Checking out connections is handled through a queue. If it\n takes longer than queue_target to get out of the queue for longer than\n queue_interval then the queue_target will be doubled and checkouts will\n start to be dropped if that target is surpassed.\n\n default: 50\nSynonyms:\npog.queue_target\npog queue_target","url":"pog.html#queue_target"},{"doc":"pog","title":"returning","content":"pub fn returning(\n  query: Query(a),\n  decoder: fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; Query(b)\n Set the decoder to use for the type of row returned by executing this\n query.\n\n If the decoder is unable to decode the row value then the query will return\n an error from the `exec` function, but the query will still have been run\n against the database.\n\nSynonyms:\npog.returning\npog returning","url":"pog.html#returning"},{"doc":"pog","title":"rows_as_map","content":"pub fn rows_as_map(config: Config, rows_as_map: Bool) -&gt; Config\n By default, pgo will return a n-tuple, in the order of the query.\n By setting `rows_as_map` to `True`, the result will be `Dict`.\nSynonyms:\npog.rows_as_map\npog rows_as_map","url":"pog.html#rows_as_map"},{"doc":"pog","title":"ssl","content":"pub fn ssl(config: Config, ssl: Bool) -&gt; Config\n Whether to use SSL or not.\n\n (default: False)\nSynonyms:\npog.ssl\npog ssl","url":"pog.html#ssl"},{"doc":"pog","title":"text","content":"pub fn text(a: String) -&gt; Value\n\nSynonyms:\npog.text\npog text","url":"pog.html#text"},{"doc":"pog","title":"time","content":"pub fn time(time: Time) -&gt; Value\n\nSynonyms:\npog.time\npog time","url":"pog.html#time"},{"doc":"pog","title":"timestamp","content":"pub fn timestamp(timestamp: Timestamp) -&gt; Value\n\nSynonyms:\npog.timestamp\npog timestamp","url":"pog.html#timestamp"},{"doc":"pog","title":"trace","content":"pub fn trace(config: Config, trace: Bool) -&gt; Config\n Trace pgo is instrumented with [OpenTelemetry][1] and\n when this option is true a span will be created (if sampled).\n\n default: False\n\n [1]: https://opentelemetry.io\nSynonyms:\npog.trace\npog trace","url":"pog.html#trace"},{"doc":"pog","title":"transaction","content":"pub fn transaction(\n  pool: Connection,\n  callback: fn(Connection) -&gt; Result(a, String),\n) -&gt; Result(a, TransactionError)\n Runs a function within a PostgreSQL transaction.\n\n If the function returns an `Ok` then the transaction is committed.\n\n If the function returns an `Error` or panics then the transaction is rolled\n back.\nSynonyms:\npog.transaction\npog transaction","url":"pog.html#transaction"},{"doc":"pog","title":"url_config","content":"pub fn url_config(database_url: String) -&gt; Result(Config, Nil)\n Parse a database url into configuration that can be used to start a pool.\nSynonyms:\npog.url_config\npog url_config","url":"pog.html#url_config"},{"doc":"pog","title":"user","content":"pub fn user(config: Config, user: String) -&gt; Config\n Username to connect to database as.\nSynonyms:\npog.user\npog user","url":"pog.html#user"},{"doc":"pog","title":"pog","content":" Postgresql client\n\n Gleam wrapper around pgo library","url":"pog.html"}]);